@@ -1,32 +1,8 @@
-/*
- * Dropbear - a SSH2 server
- * 
- * Copyright (c) 2002,2003 Matt Johnston
- * All rights reserved.
- * 
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- * 
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE. */
+#include <stdio.h>
+#include <unistd.h>
 
-/* This file (auth.c) handles authentication requests, passing it to the
- * particular type (auth-passwd, auth-pubkey). */
-
-#include "includes.h"
-#include "dbutil.h"
+#include "options.h"
+#include "util.h"
 #include "session.h"
 #include "buffer.h"
 #include "ssh.h"
@@ -34,12 +10,12 @@
 #include "auth.h"
 #include "authpasswd.h"
 #include "authpubkey.h"
+#include "pwd.h"
 
 static void authclear();
 static int checkusername(unsigned char *username, unsigned int userlen);
 static void send_msg_userauth_banner();
 
-/* initialise the first time for a session, resetting all parameters */
 void authinitialise() {
 
 	ses.authstate.failcount = 0;
@@ -47,29 +23,20 @@ void authinitialise() {
 	
 }
 
-/* Reset the auth state, but don't reset the failcount. This is for if the
- * user decides to try with a different username etc, and is also invoked
- * on initialisation */
 static void authclear() {
 	
 	ses.authstate.authdone = 0;
 	ses.authstate.pw = NULL;
-	ses.authstate.username = NULL;
-	ses.authstate.printableuser = NULL;
 	ses.authstate.authtypes = 0;
 #ifdef DROPBEAR_PUBKEY_AUTH
 	ses.authstate.authtypes |= AUTH_TYPE_PUBKEY;
 #endif
 #ifdef DROPBEAR_PASSWORD_AUTH
-	if (!ses.opts->noauthpass) {
-		ses.authstate.authtypes |= AUTH_TYPE_PASSWORD;
-	}
+	ses.authstate.authtypes |= AUTH_TYPE_PASSWORD;
 #endif
 
 }
 
-/* Send a banner message if specified to the client. The client might
- * ignore this, but possibly serves as a legal "no trespassing" sign */
 static void send_msg_userauth_banner() {
 
 	TRACE(("enter send_msg_userauth_banner"));
@@ -82,18 +49,15 @@ static void send_msg_userauth_banner() {
 
 	buf_putbyte(ses.writepayload, SSH_MSG_USERAUTH_BANNER);
 	buf_putstring(ses.writepayload, buf_getptr(ses.opts->banner,
-				ses.opts->banner->len), ses.opts->banner->len);
+				ses.opts->banner->len), ses.opts->banner->size);
 	buf_putstring(ses.writepayload, "en", 2);
 
 	encrypt_packet();
 	buf_free(ses.opts->banner);
-	ses.opts->banner = NULL;
 
 	TRACE(("leave send_msg_userauth_banner"));
 }
 
-/* handle a userauth request, check validity, pass to password or pubkey
- * checking, and handle success or failure */
 void recv_msg_userauth_request() {
 
 	unsigned char *username, *servicename, *methodname;
@@ -128,112 +92,69 @@ void recv_msg_userauth_request() {
 		dropbear_exit("unknown service in auth");
 	}
 
-	/* user wants to know what methods are supported */
 	if (methodlen == AUTH_METHOD_NONE_LEN &&
 			strncmp(methodname, AUTH_METHOD_NONE,
 				AUTH_METHOD_NONE_LEN) == 0) {
 		send_msg_userauth_failure(0, 0);
-		goto out;
-	}
-	
-	/* check username is good before continuing */
-	if (checkusername(username, userlen) == DROPBEAR_FAILURE) {
+	} else if (checkusername(username, userlen) == CHECK_USER_RETURN) {
 		/* username is invalid/no shell/etc - send failure */
 		TRACE(("sending checkusername failure"));
 		send_msg_userauth_failure(0, 1);
-		goto out;
-	}
-
 #ifdef DROPBEAR_PASSWORD_AUTH
-	if (!ses.opts->noauthpass &&
-			!(ses.opts->norootpass && ses.authstate.pw->pw_uid == 0) ) {
-		/* user wants to try password auth */
-		if (methodlen == AUTH_METHOD_PASSWORD_LEN &&
-				strncmp(methodname, AUTH_METHOD_PASSWORD,
-					AUTH_METHOD_PASSWORD_LEN) == 0) {
-			passwordauth(username, userlen);
-			goto out;
-		}
-	}
-#endif
-
+	} else if (methodlen == AUTH_METHOD_PASSWORD_LEN &&
+			strncmp(methodname, AUTH_METHOD_PASSWORD,
+				AUTH_METHOD_PASSWORD_LEN) == 0) {
+		passwordauth(username, userlen);
+#endif /* DROPBEAR_PASSWORD_AUTH */
 #ifdef DROPBEAR_PUBKEY_AUTH
-	/* user wants to try pubkey auth */
-	if (methodlen == AUTH_METHOD_PUBKEY_LEN &&
+	} else if (methodlen == AUTH_METHOD_PUBKEY_LEN &&
 			strncmp(methodname, AUTH_METHOD_PUBKEY,
 				AUTH_METHOD_PUBKEY_LEN) == 0) {
 		pubkeyauth(username, userlen);
-		goto out;
+#endif /* DROPBEAR_PUBKEY_AUTH */
+	} else {
+		send_msg_userauth_failure(0, 1);
 	}
-#endif
-
-	/* nothing matched, we just fail */
-	send_msg_userauth_failure(0, 1);
-
-out:
 
 	m_free(username);
 	m_free(servicename);
 	m_free(methodname);
 }
 
-/* Check that the username exists, has a non-empty password, and has a valid
- * shell.
- * returns DROPBEAR_SUCCESS on valid username, DROPBEAR_FAILURE on failure */
 static int checkusername(unsigned char *username, unsigned int userlen) {
 
 	char* shell;
 	
 	TRACE(("enter checkusername"));
 	if (userlen > MAX_USERNAME_LEN) {
-		return DROPBEAR_FAILURE;
+		return CHECK_USER_RETURN;
 	}
 
 	/* new user or username has changed */
-	if (ses.authstate.username == NULL ||
-		strcmp(username, ses.authstate.username) != 0) {
+	if (ses.authstate.pw == NULL ||
+		strcmp(username, ses.authstate.pw->pw_name) != 0) {
 			/* the username needs resetting */
-			if (ses.authstate.username != NULL) {
-				dropbear_log(LOG_WARNING, "client trying multiple usernames");
-				m_free(ses.authstate.username);
-			}
 			authclear();
 			ses.authstate.pw = getpwnam((char*)username);
-			ses.authstate.username = strdup(username);
-			m_free(ses.authstate.printableuser);
 	}
 
 	/* check that user exists */
 	if (ses.authstate.pw == NULL) {
-		TRACE(("leave checkusername: user '%s' doesn't exist", username));
-		dropbear_log(LOG_WARNING,
-				"login attempt for nonexistent user");
+		TRACE(("leave checkusername: user doesn't exist"));
 		send_msg_userauth_failure(0, 1);
-		return DROPBEAR_FAILURE;
-	}
-
-	/* We can set it once we know its a real user */
-	ses.authstate.printableuser = strdup(ses.authstate.pw->pw_name);
-
-	/* check for non-root if desired */
-	if (ses.opts->norootlogin && ses.authstate.pw->pw_uid == 0) {
-		TRACE(("leave checkusername: root login disabled"));
-		dropbear_log(LOG_WARNING, "root login rejected");
-		send_msg_userauth_failure(0, 1);
-		return DROPBEAR_FAILURE;
+		return CHECK_USER_RETURN;
 	}
 
 	/* check for an empty password */
 	if (ses.authstate.pw->pw_passwd[0] == '\0') {
 		TRACE(("leave checkusername: empty pword"));
-		dropbear_log(LOG_WARNING, "user '%s' has blank password, rejected",
-				ses.authstate.printableuser);
 		send_msg_userauth_failure(0, 1);
-		return DROPBEAR_FAILURE;
+		return CHECK_USER_RETURN;
 	}
 
 	TRACE(("shell is %s", ses.authstate.pw->pw_shell));
 	/* check that the shell is valid */
+	/* XXX - todo check this is correct: empty shell is ok */
 	if (ses.authstate.pw->pw_shell[0] == '\0') {
 		goto goodshell;
 	}
@@ -248,26 +169,22 @@ static int checkusername(unsigned char *username, unsigned int userlen) {
 	/* no matching shell */
 	endusershell();
 	TRACE(("no matching shell"));
-	dropbear_log(LOG_WARNING, "user '%s' has invalid shell, rejected",
-				ses.authstate.printableuser);
 	send_msg_userauth_failure(0, 1);
-	return DROPBEAR_FAILURE;
+	return CHECK_USER_RETURN;
 	
 goodshell:
 	endusershell();
 	TRACE(("matching shell"));
 
-	TRACE(("uid = %d", ses.authstate.pw->pw_uid));
+	TRACE(("uid = %d\n", ses.authstate.pw->pw_uid));
 	TRACE(("leave checkusername"));
-	return DROPBEAR_SUCCESS;
+	return CHECK_USER_CONTINUE;
 
 }
 
-/* Send a failure message to the client, in responds to a userauth_request.
- * Partial indicates whether to set the "partial success" flag,
+/* partial indicates whether to set the "partial success" flag,
  * incrfail is whether to count this failure in the failure count (which
- * is limited. This function also handles disconnection after too many
- * failures */
+ * is limited */
 void send_msg_userauth_failure(int partial, int incrfail) {
 
 	buffer *typebuf;
@@ -301,27 +218,20 @@ void send_msg_userauth_failure(int partial, int incrfail) {
 	encrypt_packet();
 
 	if (incrfail) {
-		usleep(300000); /* XXX improve this */
 		ses.authstate.failcount++;
+		sleep(FAIL_SLEEP_TIME);
 	}
 
 	if (ses.authstate.failcount >= MAX_AUTH_TRIES) {
-		char * userstr;
 		/* XXX - send disconnect ? */
 		TRACE(("Max auth tries reached, exiting"));
-
-		if (ses.authstate.printableuser == NULL) {
-			userstr = "is invalid";
-		} else {
-			userstr = ses.authstate.printableuser;
-		}
-		dropbear_exit("Max auth tries reached - user %s", userstr);
+		exit(0);
 	}
 	
 	TRACE(("leave send_msg_userauth_failure"));
 }
 
-/* Send a success message to the user, and set the "authdone" flag */
+/* only to be called when writebuf is empty */
 void send_msg_userauth_success() {
 
 	TRACE(("enter send_msg_userauth_success"));
@@ -332,7 +242,7 @@ void send_msg_userauth_success() {
 	encrypt_packet();
 
 	ses.authstate.authdone = 1;
-	close(ses.childpipe); /* remove from the list of pre-auth sockets */
+	close(ses.childpipe);
 	TRACE(("leave send_msg_userauth_success"));
 
 }
