@@ -1,109 +1,75 @@
-/*
- * Dropbear - a SSH2 server
- * 
- * Copyright (c) 2002,2003 Matt Johnston
- * All rights reserved.
- * 
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- * 
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE. */
-
-#include "includes.h"
-#include "dbutil.h"
+#include "options.h"
+#include "util.h"
 #include "bignum.h"
 #include "dss.h"
 #include "buffer.h"
 #include "ssh.h"
 #include "random.h"
 
-/* Handle DSS (Digital Signature Standard), aka DSA (D.S. Algorithm),
- * operations, such as key reading, signing, verification. Key generation
- * is in gendss.c, since it isn't required in the server itself.
- *
- * See FIPS186 or the Handbook of Applied Cryptography for details of the
- * algorithm */
+#include "libtomcrypt/mycrypt.h"
 
 #ifdef DROPBEAR_DSS 
 
 /* Load a dss key from a buffer, initialising the values.
  * The key will have the same format as buf_put_dss_key.
  * These should be freed with dss_key_free.
- * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
+ * Returns 0 on fail, -1 on success */
 int buf_get_dss_pub_key(buffer* buf, dss_key *key) {
 
-	TRACE(("enter buf_get_dss_pub_key"))
-	dropbear_assert(key != NULL);
+	assert(key != NULL);
 	key->p = m_malloc(sizeof(mp_int));
+	m_mp_init(key->p);
 	key->q = m_malloc(sizeof(mp_int));
+	m_mp_init(key->q);
 	key->g = m_malloc(sizeof(mp_int));
+	m_mp_init(key->g);
 	key->y = m_malloc(sizeof(mp_int));
-	m_mp_init_multi(key->p, key->q, key->g, key->y, NULL);
+	m_mp_init(key->y);
 	key->x = NULL;
 
 	buf_incrpos(buf, 4+SSH_SIGNKEY_DSS_LEN); /* int + "ssh-dss" */
-	if (buf_getmpint(buf, key->p) == DROPBEAR_FAILURE
-	 || buf_getmpint(buf, key->q) == DROPBEAR_FAILURE
-	 || buf_getmpint(buf, key->g) == DROPBEAR_FAILURE
-	 || buf_getmpint(buf, key->y) == DROPBEAR_FAILURE) {
-		TRACE(("leave buf_get_dss_pub_key: failed reading mpints"))
-		return DROPBEAR_FAILURE;
+	if (buf_getmpint(buf, key->p) != 0
+	 || buf_getmpint(buf, key->q) != 0
+	 || buf_getmpint(buf, key->g) != 0
+	 || buf_getmpint(buf, key->y) != 0) {
+		dropbear_msg("failure reading dss pubkey");
+		return -1;
 	}
 
-	if (mp_count_bits(key->p) < MIN_DSS_KEYLEN) {
-		dropbear_log(LOG_WARNING, "DSS key too short");
-		TRACE(("leave buf_get_dss_pub_key: short key"))
-		return DROPBEAR_FAILURE;
-	}
-
-	TRACE(("leave buf_get_dss_pub_key: success"))
-	return DROPBEAR_SUCCESS;
+	return 0;
 }
 
-/* Same as buf_get_dss_pub_key, but reads a private "x" key at the end.
- * Loads a private dss key from a buffer
- * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
+/* same as buf_get_dss_pub_key, but reads a private "x" key at the end.
+ * Loads a private dss key from a buffer */
 int buf_get_dss_priv_key(buffer* buf, dss_key *key) {
 
-	int ret = DROPBEAR_FAILURE;
+	int ret = 0;
 
-	dropbear_assert(key != NULL);
+	assert(key != NULL);
 
 	ret = buf_get_dss_pub_key(buf, key);
-	if (ret == DROPBEAR_FAILURE) {
-		return DROPBEAR_FAILURE;
+	if (ret != 0) {
+		goto out;
 	}
 
 	key->x = m_malloc(sizeof(mp_int));
 	m_mp_init(key->x);
 	ret = buf_getmpint(buf, key->x);
-	if (ret == DROPBEAR_FAILURE) {
-		m_free(key->x);
+out:
+	
+	if (ret != 0) {
+		dropbear_msg("failure reading dss privkey");
 	}
-
 	return ret;
 }
 	
 
-/* Clear and free the memory used by a public or private key */
+/* clear and free the memory used by a public key */
 void dss_key_free(dss_key *key) {
 
-	TRACE(("enter dsa_key_free"))
+	TRACE(("enter dsa_key_free"));
 	if (key == NULL) {
-		TRACE(("enter dsa_key_free: key == NULL"))
+		TRACE(("enter dsa_key_free: key == NULL"));
 		return;
 	}
 	if (key->p) {
@@ -127,10 +93,10 @@ void dss_key_free(dss_key *key) {
 		m_free(key->x);
 	}
 	m_free(key);
-	TRACE(("leave dsa_key_free"))
+	TRACE(("leave dsa_key_free"));
 }
 
-/* put the dss public key into the buffer in the required format:
+/* put the dss key into the buffer in the required format:
  *
  * string	"ssh-dss"
  * mpint	p
@@ -140,7 +106,7 @@ void dss_key_free(dss_key *key) {
  */
 void buf_put_dss_pub_key(buffer* buf, dss_key *key) {
 
-	dropbear_assert(key != NULL);
+	assert(key != NULL);
 	buf_putstring(buf, SSH_SIGNKEY_DSS, SSH_SIGNKEY_DSS_LEN);
 	buf_putmpint(buf, key->p);
 	buf_putmpint(buf, key->q);
@@ -152,32 +118,26 @@ void buf_put_dss_pub_key(buffer* buf, dss_key *key) {
 /* Same as buf_put_dss_pub_key, but with the private "x" key appended */
 void buf_put_dss_priv_key(buffer* buf, dss_key *key) {
 
-	dropbear_assert(key != NULL);
+	assert(key != NULL);
 	buf_put_dss_pub_key(buf, key);
 	buf_putmpint(buf, key->x);
 
 }
 
 #ifdef DROPBEAR_SIGNKEY_VERIFY
-/* Verify a DSS signature (in buf) made on data by the key given. 
- * returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
+/* returns 1 if the signature verifies, 0 otherwise */
 int buf_dss_verify(buffer* buf, dss_key *key, const unsigned char* data,
 		unsigned int len) {
 
 	unsigned char msghash[SHA1_HASH_SIZE];
 	hash_state hs;
-	int ret = DROPBEAR_FAILURE;
-	DEF_MP_INT(val1);
-	DEF_MP_INT(val2);
-	DEF_MP_INT(val3);
-	DEF_MP_INT(val4);
+	int ret = 0;
+	mp_int val1, val2, val3, val4;
 	char * string = NULL;
 	int stringlen;
 
-	TRACE(("enter buf_dss_verify"))
-	dropbear_assert(key != NULL);
-
-	m_mp_init_multi(&val1, &val2, &val3, &val4, NULL);
+	TRACE(("enter buf_dss_verify"));
+	assert(key != NULL);
 
 	/* get blob, check length */
 	string = buf_getstring(buf, &stringlen);
@@ -190,13 +150,16 @@ int buf_dss_verify(buffer* buf, dss_key *key, const unsigned char* data,
 	sha1_process(&hs, data, len);
 	sha1_done(&hs, msghash);
 
+	m_mp_init(&val1);
+	m_mp_init(&val2);
+	m_mp_init(&val3);
+	m_mp_init(&val4);
+
 	/* create the signature - s' and r' are the received signatures in buf */
 	/* w = (s')-1 mod q */
 	/* let val1 = s' */
-	bytes_to_mp(&val1, &string[SHA1_HASH_SIZE], SHA1_HASH_SIZE);
-
-	if (mp_cmp(&val1, key->q) != MP_LT) {
-		TRACE(("verify failed, s' >= q"))
+	if (mp_read_unsigned_bin(&val1, &string[SHA1_HASH_SIZE], SHA1_HASH_SIZE)
+			!= MP_OKAY) {
 		goto out;
 	}
 	/* let val2 = w = (s')^-1 mod q*/
@@ -206,8 +169,9 @@ int buf_dss_verify(buffer* buf, dss_key *key, const unsigned char* data,
 
 	/* u1 = ((SHA(M')w) mod q */
 	/* let val1 = SHA(M') = msghash */
-	bytes_to_mp(&val1, msghash, SHA1_HASH_SIZE);
-
+	if (mp_read_unsigned_bin(&val1, msghash, SHA1_HASH_SIZE) != MP_OKAY) {
+		goto out;
+	}
 	/* let val3 = u1 = ((SHA(M')w) mod q */
 	if (mp_mulmod(&val1, &val2, key->q, &val3) != MP_OKAY) {
 		goto out;
@@ -215,9 +179,8 @@ int buf_dss_verify(buffer* buf, dss_key *key, const unsigned char* data,
 
 	/* u2 = ((r')w) mod q */
 	/* let val1 = r' */
-	bytes_to_mp(&val1, &string[0], SHA1_HASH_SIZE);
-	if (mp_cmp(&val1, key->q) != MP_LT) {
-		TRACE(("verify failed, r' >= q"))
+	if (mp_read_unsigned_bin(&val1, &string[0], SHA1_HASH_SIZE)
+			!= MP_OKAY) {
 		goto out;
 	}
 	/* let val4 = u2 = ((r')w) mod q */
@@ -244,13 +207,16 @@ int buf_dss_verify(buffer* buf, dss_key *key, const unsigned char* data,
 	}
 	
 	/* check whether signatures verify */
-	if (mp_cmp(&val2, &val1) == MP_EQ) {
+	if (mp_cmp(&val2, &val1) == 0) {
 		/* good sig */
-		ret = DROPBEAR_SUCCESS;
+		ret = 1;
 	}
 
 out:
-	mp_clear_multi(&val1, &val2, &val3, &val4, NULL);
+	mp_clear(&val1);
+	mp_clear(&val2);
+	mp_clear(&val3);
+	mp_clear(&val4);
 	m_free(string);
 
 	return ret;
@@ -258,32 +224,11 @@ out:
 }
 #endif /* DROPBEAR_SIGNKEY_VERIFY */
 
-#ifdef DSS_PROTOK	
-/* convert an unsigned mp into an array of bytes, malloced.
- * This array must be freed after use, len contains the length of the array,
- * if len != NULL */
-static unsigned char* mptobytes(mp_int *mp, int *len) {
-	
-	unsigned char* ret;
-	int size;
-
-	size = mp_unsigned_bin_size(mp);
-	ret = m_malloc(size);
-	if (mp_to_unsigned_bin(mp, ret) != MP_OKAY) {
-		dropbear_exit("mem alloc error");
-	}
-	if (len != NULL) {
-		*len = size;
-	}
-	return ret;
-}
-#endif
-
-/* Sign the data presented with key, writing the signature contents
+/* sign the data presented in len with key, writing the signature contents
  * to the buffer
  *
- * When DSS_PROTOK is #defined:
- * The alternate k generation method is based on the method used in PuTTY. 
+ * when DSS_PROTOK is #defined:
+ * The alternate k generation method is based on the method used in putty. 
  * In particular to avoid being vulnerable to attacks using flaws in random
  * generation of k, we use the following:
  *
@@ -302,26 +247,24 @@ void buf_put_dss_sign(buffer* buf, dss_key *key, const unsigned char* data,
 	unsigned char privkeyhash[SHA512_HASH_SIZE];
 	unsigned char *privkeytmp;
 	unsigned char proto_k[SHA512_HASH_SIZE];
-	DEF_MP_INT(dss_protok);
+	mp_int dss_protok;
+#else
+	unsigned char kbuf[SHA1_HASH_SIZE];
 #endif
-	DEF_MP_INT(dss_k);
-	DEF_MP_INT(dss_m);
-	DEF_MP_INT(dss_temp1);
-	DEF_MP_INT(dss_temp2);
-	DEF_MP_INT(dss_r);
-	DEF_MP_INT(dss_s);
+	mp_int dss_k, dss_m;
+	mp_int dss_temp1, dss_temp2;
+	mp_int dss_r, dss_s;
 	hash_state hs;
 	
-	TRACE(("enter buf_put_dss_sign"))
-	dropbear_assert(key != NULL);
+	TRACE(("enter buf_put_dss_sign"));
+	assert(key != NULL);
 	
 	/* hash the data */
 	sha1_init(&hs);
 	sha1_process(&hs, data, len);
 	sha1_done(&hs, msghash);
 
-	m_mp_init_multi(&dss_k, &dss_temp1, &dss_temp2, &dss_r, &dss_s,
-			&dss_m, NULL);
+	m_mp_init(&dss_k);
 #ifdef DSS_PROTOK	
 	/* hash the privkey */
 	privkeytmp = mptobytes(key->x, &i);
@@ -340,18 +283,27 @@ void buf_put_dss_sign(buffer* buf, dss_key *key, const unsigned char* data,
 
 	/* generate k */
 	m_mp_init(&dss_protok);
-	bytes_to_mp(&dss_protok, proto_k, SHA512_HASH_SIZE);
-	if (mp_mod(&dss_protok, key->q, &dss_k) != MP_OKAY) {
-		dropbear_exit("dss error");
-	}
+	bytestomp(&dss_protok, proto_k, SHA512_HASH_SIZE);
+	mp_mod(&dss_protok, key->q, &dss_k);
 	mp_clear(&dss_protok);
 	m_burn(proto_k, SHA512_HASH_SIZE);
 #else /* DSS_PROTOK not defined*/
-	gen_random_mpint(key->q, &dss_k);
+	do {
+		genrandom(kbuf, SHA1_HASH_SIZE);
+		if (mp_read_unsigned_bin(&dss_k, kbuf, SHA1_HASH_SIZE) != MP_OKAY) {
+			dropbear_exit("dss error");
+		}
+	} while (mp_cmp(&dss_k, key->q) >= 0 || mp_cmp_d(&dss_k, 0) <= 0);
+	m_burn(kbuf, SHA1_HASH_SIZE);
 #endif
 
 	/* now generate the actual signature */
-	bytes_to_mp(&dss_m, msghash, SHA1_HASH_SIZE);
+	m_mp_init(&dss_temp1);
+	m_mp_init(&dss_temp2);
+	m_mp_init(&dss_r);
+	m_mp_init(&dss_s);
+	m_mp_init(&dss_m);
+	bytestomp(&dss_m, msghash, SHA1_HASH_SIZE);
 
 	/* g^k mod p */
 	if (mp_exptmod(key->g, &dss_k, key->p, &dss_temp1) !=  MP_OKAY) {
@@ -370,22 +322,27 @@ void buf_put_dss_sign(buffer* buf, dss_key *key, const unsigned char* data,
 	if (mp_addmod(&dss_m, &dss_temp1, key->q, &dss_temp2) != MP_OKAY) {
 		dropbear_exit("dss error");
 	}
+	mp_clear(&dss_m);
 	
 	/* (k^-1) mod q */
 	if (mp_invmod(&dss_k, key->q, &dss_temp1) != MP_OKAY) {
 		dropbear_exit("dss error");
 	}
+	mp_clear(&dss_k);
 
 	/* s = (k^-1(SHA1(M) + xr)) mod q */
 	if (mp_mulmod(&dss_temp1, &dss_temp2, key->q, &dss_s) != MP_OKAY) {
 		dropbear_exit("dss error");
 	}
-
+	mp_clear(&dss_temp1);
+	mp_clear(&dss_temp2);
+	
+	/* create the signature to return */
 	buf_putstring(buf, SSH_SIGNKEY_DSS, SSH_SIGNKEY_DSS_LEN);
 	buf_putint(buf, 2*SHA1_HASH_SIZE);
 
 	writelen = mp_unsigned_bin_size(&dss_r);
-	dropbear_assert(writelen <= SHA1_HASH_SIZE);
+	assert(writelen <= SHA1_HASH_SIZE);
 	/* need to pad to 160 bits with leading zeros */
 	for (i = 0; i < SHA1_HASH_SIZE - writelen; i++) {
 		buf_putbyte(buf, 0);
@@ -398,7 +355,7 @@ void buf_put_dss_sign(buffer* buf, dss_key *key, const unsigned char* data,
 	buf_incrwritepos(buf, writelen);
 
 	writelen = mp_unsigned_bin_size(&dss_s);
-	dropbear_assert(writelen <= SHA1_HASH_SIZE);
+	assert(writelen <= SHA1_HASH_SIZE);
 	/* need to pad to 160 bits with leading zeros */
 	for (i = 0; i < SHA1_HASH_SIZE - writelen; i++) {
 		buf_putbyte(buf, 0);
@@ -410,12 +367,7 @@ void buf_put_dss_sign(buffer* buf, dss_key *key, const unsigned char* data,
 	mp_clear(&dss_s);
 	buf_incrwritepos(buf, writelen);
 
-	mp_clear_multi(&dss_k, &dss_temp1, &dss_temp2, &dss_r, &dss_s,
-			&dss_m, NULL);
-	
-	/* create the signature to return */
-
-	TRACE(("leave buf_put_dss_sign"))
+	TRACE(("leave buf_put_dss_sign"));
 }
 
 #endif /* DROPBEAR_DSS */
